/* PM 2017
 * Radu Ioana-Roxana
 * 
 * Implementarea functiilor necesare comunicarii cu accelerometrul LSM6DS33
 *
 */

#include "lsm6.h"
#include "spi.h"
#include "usart.h"
#include <avr/io.h>

/* Scrie in registrul reg valoarea value.*/
void write_reg(uint8_t reg, uint8_t value)
{
	uint8_t data;
	
	/* trimite primul octet */
	data = reg;
	
	/* setam cel mai semnificativ bit 0 pentru scriere (vezi datasheet) */
	data |= (0 << 7);
	
	/* incepe transferul. pune CS pe low */
	PORTB &= ~(1 << PB4);
	
	SPI_exchange(data);
	SPI_exchange(value);
	
	/* transfer terminat. pune CS high */
	PORTB |= (1 << PB4);
}

/* Citeste un registru si returneaza valoarea din acesta */
uint8_t read_reg(uint8_t reg)
{
	uint8_t data;
	uint8_t result = 0;
	
	/* Pregatim primul octet de trimis */
	data = reg;
	/* setam primul bit pe 1 pentru a anunta intentia de citire (pagina 32 datasheet) */
	data |= (1 << 7);

	/* incepe transferul. pune CS pe low */
	PORTB &= ~(1 << PB4);
	
	SPI_exchange(data);
	result = SPI_exchange(data);
	
	/* transfer terminat. pune CS high */
	PORTB |= (1 << PB4);

	return result;
}

/* Initializeaza accelerometrul/giroscopul */
void init_lsm6()
{
	DDRB |= (1 << PB4);
	PORTB |= (1 << PB4);

	/* initializeaza comunicatia prin SPI*/
	SPI_init();
	
	/* activare accelerometru conform pg28 paragraf 3.1 */
	
	/* axele X, Y, Z pornite (enabled) */
	write_reg(CTRL1_XL, 0x60);
	/* mod high performance 416Hz*/
	write_reg(CTRL9_XL, 0x38);
	
	/* activare giroscop similar cu accelerometrul*/
	write_reg(CTRL3_C, 0x04);
	write_reg(CTRL2_G, 0x60);
}

/* Citeste toate cele 3 axe ale accelerometrului. Pentru fiecare axa valoarea este pe 2 octeti: un octet high si unul low */
void read_acc(uint8_t *xlow, uint8_t *xhigh, uint8_t *ylow, uint8_t *yhigh, uint8_t *zlow, uint8_t *zhigh)
{
	*xlow = read_reg(OUTX_L_XL);
	*xhigh = read_reg(OUTX_H_XL);
	
	*ylow = read_reg(OUTY_L_XL);
	*yhigh = read_reg(OUTY_H_XL);

	*zlow = read_reg(OUTZ_L_XL);
	*zhigh = read_reg(OUTZ_H_XL);
}

/* Citeste toate cele 3 axe ale giroscopului. Pentru fiecare axa valoarea este pe 2 octeti: un octet high si unul low*/
void read_gyro(uint8_t *xlow, uint8_t *xhigh, uint8_t *ylow, uint8_t *yhigh, uint8_t *zlow, uint8_t *zhigh)
{
	*xlow = read_reg(OUTX_L_G);
	*xhigh = read_reg(OUTX_H_G);
	
	*ylow = read_reg(OUTY_L_G);
	*yhigh = read_reg(OUTY_H_G);
	
	*zlow = read_reg(OUTZ_L_G);
	*zhigh = read_reg(OUTZ_H_G);	
}
